use std::time::{SystemTime, UNIX_EPOCH};

use serde::{Deserialize, Serialize};

#[derive(Serialize, Deserialize, Debug, Clone)]
pub struct RunHistory {
    entries: Vec<TrackEntry>,

    #[serde(flatten)]
    metadata: RunHistoryMetadata,
}

impl RunHistory {
    pub fn uuid(&self) -> String {
        self.metadata.uuid.clone()
    }
}

impl From<RunHistory> for RunHistoryMetadata {
    fn from(history: RunHistory) -> Self {
        history.metadata
    }
}

#[derive(Serialize, Deserialize, Debug, Clone)]
pub enum EntryType {
    PlanForceNext,
    PlanForcePrev,
    ZoneEnter,
    CheckpointReached,
}

#[derive(Serialize, Deserialize, Debug, Clone)]
pub struct TrackEntry {
    #[serde(rename = "type")]
    entry_type: EntryType,
    zone: String,
    current_elapsed_millis: u64,
}

#[derive(Serialize, Deserialize, Debug, Clone)]
pub struct RunHistoryMetadata {
    uuid: String,
    current_elapsed_millis: u64,
    associated_name: String,
    #[serde(default = "default_last_updated_date_now")]
    last_updated: u64,
}

fn default_last_updated_date_now() -> u64 {
    SystemTime::now()
        .duration_since(UNIX_EPOCH)
        .unwrap()
        .as_millis() as u64
}
