import { Injectable, NgZone } from '@angular/core';
import { ShortcutService } from './shortcut.service';
import { EventsService } from './events.service';
import { Event } from "@tauri-apps/api/event";
import { ConfigService } from './config.service';
import { webviewWindow as appWindow } from '@tauri-apps/api';

import { Subscription } from 'rxjs';
import { invoke } from '@tauri-apps/api/core';

export class StateEvent {
  Visible?: any;
  Interactable?: any;
  Hidden?: any;
}

@Injectable({
  providedIn: 'root'
})
export class OverlayService {
  interactable: boolean = false;
  isBinding: boolean = false;
  visible: boolean = false;
  isOverlay: boolean;

  constructor(private shortcuts: ShortcutService, private events: EventsService, private configService: ConfigService) {
    if (appWindow.getCurrent().label == "Overlay") {

      this.registerInitialBinds();

      this.events.listen<StateEvent>("OverlayStateChange").subscribe(this.onOverlayStateChange.bind(this));
    }
    this.isOverlay = appWindow.getCurrent().label === "Overlay";
  }

  registerInitialBinds(retry: boolean = true) {
    this.shortcuts.register(this.configService.config.toggleOverlay).subscribe({
      next: (_shortcut) => {
        this.onToggleOverlay()
      },
      error: (err) => {
        console.error("error binding overlay toggle", err);
        this.shortcuts.force_unbind(this.configService.config.toggleOverlay).subscribe(() => this.registerInitialBinds(false))
      }
    });
  }

  onOverlayStateChange(event: Event<StateEvent>) {
    console.log("overlay state change: ", event);
    this.interactable = event.payload.Interactable != null;
    if (event.payload.Hidden) { 
      this.visible = false;
      console.log("visible: ", this.visible);
     } else { 
      this.visible = true;
      console.log("visible: ", this.visible);
     };
  }

  setInteractable() {
    invoke("set_interactable", { interactable: true }).then();
  }

  onToggleOverlay() {
    if (this.interactable) {
      this.interactable = false;
    } else {
      this.interactable = true;
    }
    console.log("Set interactable: ", this.interactable);
    invoke("set_interactable", { interactable: this.interactable }).then();
  }
}
